package flabbergast;
import java.util.Arrays;
import java.util.Properties;
import java.util.regex.Pattern;
class JdbcParser {
public static String parse(String provider, String uri_fragment, Properties in_properties, Properties out_properties, ResourcePathFinder finder, Ptr<String> err) {
if (provider.equals("mssql")) {
	String db_fragment = JdbcUriHandler.parseUri(uri_fragment, out_properties, "user", "password", "databaseName", err);
	if(db_fragment == null) return null;
	if (!Arrays.asList().containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	return "jdbc:sqlserver:"+ "//" + db_fragment;
}
if (provider.equals("mysql")) {
	String db_fragment = JdbcUriHandler.parseUri(uri_fragment, out_properties, "user", "password", null, err);
	if(db_fragment == null) return null;
	if (!Arrays.asList().containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	out_properties.setProperty("characterEncoding", "UTF-8");
	return "jdbc:mysql:"+ "//" + db_fragment;
}
if (provider.equals("oracle+host")) {
	String db_fragment = JdbcUriHandler.parseUri(uri_fragment, out_properties, "user", "password", null, err);
	if(db_fragment == null) return null;
	String[] outer = db_fragment.split("/");
	String[] inner = outer[0].split(":");
	if (!Arrays.asList().containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	return "jdbc:oracle:oci:@"+ String.format("descriptor", inner[0], inner.length > 1 ? inner[1] : "1521", outer[1]);
}
if (provider.equals("oracle+tns")) {

if (!Pattern.matches("^[A-Za-z0-9_]+$", uri_fragment)) { err.set("Invalid file specified."); return null; }
	if (!Arrays.asList().containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	return "jdbc:oracle:oci:@" + uri_fragment;
}
if (provider.equals("postgresql")) {
	String db_fragment = JdbcUriHandler.parseUri(uri_fragment, out_properties, "user", "password", null, err);
	if(db_fragment == null) return null;
	if (!Arrays.asList("ssl").containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	if (in_properties.containsKey("ssl")) {
	if (!Pattern.matches("^(true|false)$", in_properties.getProperty("ssl"))) { err.set("The value for “ssl” is not allowed."); return null; }
		out_properties.setProperty("ssl", in_properties.getProperty("ssl"));
	}
	return "jdbc:postgresql:"+ "//" + db_fragment;
}
if (provider.equals("sqlite")) {

if (!Pattern.matches(".*", uri_fragment)) { err.set("Invalid file specified."); return null; }
	if (!Arrays.asList().containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	return "jdbc:sqlite:" + uri_fragment;
}
if (provider.equals("sqlite+res")) {

if (!Pattern.matches("^[A-Za-z0-9/]*$", uri_fragment)) { err.set("Invalid resource file specified."); return null; }
java.util.List<java.io.File> f0000000030_files = finder.findAll(uri_fragment, ".sqlite", ".sqlite3", ".db", ".db3", ".s3db");
if(f0000000030_files.size() == 0) { err.set("Cannot find resource “" + uri_fragment + "”."); return null; }
	if (!Arrays.asList().containsAll(in_properties.stringPropertyNames())) { err.set("Unknown parameters specified."); return null; }
	return "jdbc:sqlite:" + f0000000030_files.get(0).getAbsolutePath();
}

err.set(String.format("Unknown database provider “%s”.", provider));
return null;
}
}
